@@ -236,14 +236,14 @@ html_object_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed
 static gint
 calc_min_width (HTMLObject *o, HTMLPainter *painter)
 {
-	html_object_calc_size (o, painter, FALSE);
+	html_object_calc_size (o, painter, NULL);
 	return o->width;
 }
 
 static gint
 calc_preferred_width (HTMLObject *o, HTMLPainter *painter)
 {
-	html_object_calc_size (o, painter, FALSE);
+	html_object_calc_size (o, painter, NULL);
 	return o->width;
 }
 	
@@ -380,7 +380,7 @@ relayout (HTMLObject *self,
 		self->y -= prev_ascent + prev_descent;
 	}
 
-	changed = html_object_calc_size (self, engine->painter, FALSE);
+	changed = html_object_calc_size (self, engine->painter, NULL);
 
 	if (prev_width == self->width
 	    && prev_ascent == self->ascent
@@ -1239,6 +1239,10 @@ html_object_get_cursor (HTMLObject *self,
 			gint *x2, gint *y2)
 {
 	(* HO_CLASS (self)->get_cursor) (self, painter, offset, x1, y1, x2, y2);
+
+	if (*y2 - *y1 > *y2 - self->ascent)
+		*y2 = *y1 + 20;
+
 	if (!html_object_is_text (self) && *y2 - *y1 < 10) {
 		gint missing = 10 - (*y2 - *y1);
 
@@ -1676,7 +1680,7 @@ move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObje
 			if (html_object_accepts_cursor (obj))
 				break;
 			last_obj = obj;
-			if (obj = move_object_downtree_cursor (obj, down_fn, next_fn))
+			if ((obj = move_object_downtree_cursor (obj, down_fn, next_fn)))
 				break;
 			obj = last_obj;
 		}
